@@ -233,3 +233,82 @@ class SortedManyToManyField(ManyToManyField):
             defaults['form_class'] = SortedMultipleChoiceField
         defaults.update(kwargs)
         return super(SortedManyToManyField, self).formfield(**defaults)
+
+
+# Add introspection rules for South database migrations
+# See http://south.aeracode.org/docs/customfields.html
+try:
+    import south
+except ImportError:
+    south = None
+
+if south is not None:
+    from south.modelsinspector import add_introspection_rules
+    add_introspection_rules(
+        [(
+            (SortedManyToManyField,),
+            [],
+            {"sorted": ["sorted", {"default": True}]},
+        )],
+        [r'^sortedm2m\.fields\.SortedManyToManyField']
+    )
+
+    # Monkeypatch South M2M actions to create the sorted through model.
+    # FIXME: This doesn't detect if you changed the sorted argument to the field.
+    import south.creator.actions
+    from south.creator.freezer import model_key
+
+    class AddM2M(south.creator.actions.AddM2M):
+        SORTED_FORWARDS_TEMPLATE = '''
+        # Adding SortedM2M table for field %(field_name)s on '%(model_name)s'
+        db.create_table(%(table_name)r, (
+            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
+            (%(left_field)r, models.ForeignKey(orm[%(left_model_key)r], null=False)),
+            (%(right_field)r, models.ForeignKey(orm[%(right_model_key)r], null=False)),
+            (%(sort_field)r, models.IntegerField())
+        ))
+        db.create_unique(%(table_name)r, [%(left_column)r, %(right_column)r])'''
+
+        def console_line(self):
+            if isinstance(self.field, SortedManyToManyField) and self.field.sorted:
+                return " + Added SortedM2M table for %s on %s.%s" % (
+                    self.field.name,
+                    self.model._meta.app_label,
+                    self.model._meta.object_name,
+                )
+            else:
+                return super(AddM2M, self).console_line()
+
+        def forwards_code(self):
+            if isinstance(self.field, SortedManyToManyField) and self.field.sorted:
+                return self.SORTED_FORWARDS_TEMPLATE % {
+                    "model_name": self.model._meta.object_name,
+                    "field_name": self.field.name,
+                    "table_name": self.field.m2m_db_table(),
+                    "left_field": self.field.m2m_column_name()[:-3], # Remove the _id part
+                    "left_column": self.field.m2m_column_name(),
+                    "left_model_key": model_key(self.model),
+                    "right_field": self.field.m2m_reverse_name()[:-3], # Remove the _id part
+                    "right_column": self.field.m2m_reverse_name(),
+                    "right_model_key": model_key(self.field.rel.to),
+                    "sort_field": SORT_VALUE_FIELD_NAME,
+                }
+            else:
+                return super(AddM2M, self).forwards_code()
+
+    class DeleteM2M(AddM2M):
+        def console_line(self):
+            return " - Deleted M2M table for %s on %s.%s" % (
+                self.field.name,
+                self.model._meta.app_label,
+                self.model._meta.object_name,
+            )
+
+        def forwards_code(self):
+            return AddM2M.backwards_code(self)
+
+        def backwards_code(self):
+            return AddM2M.forwards_code(self)
+
+    south.creator.actions.AddM2M = AddM2M
+    south.creator.actions.DeleteM2M = DeleteM2M
